@@ -1,4 +1,3 @@
-/* $Id: terminal-preferences.c 30325 2009-07-16 18:36:27Z nick $ */
 /*-
  * Copyright (c) 2004-2007 os-cillation e.K.
  *
@@ -1437,7 +1436,7 @@ terminal_preferences_check_blurb (GParamSpec *spec)
 
   /* check if the generated option name is equal to the blurb */
   if (!exo_str_is_equal (option, g_param_spec_get_blurb (spec)))
-    g_critical ("Blurb does not match option name %s", name);
+    g_error ("The blurb of property \"%s\" does not match option name", name);
 
   /* cleanup */
   g_free (option);
@@ -1460,13 +1459,12 @@ terminal_preferences_load (TerminalPreferences *preferences)
   guint         n;
 
   filename = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, "Terminal/terminalrc");
+  if (G_UNLIKELY (filename == NULL))
+    return;
+
   rc = xfce_rc_simple_open (filename, TRUE);
   if (G_UNLIKELY (rc == NULL))
-    {
-      g_warning ("Unable to load terminal preferences.");
-      g_free (filename);
-      return;
-    }
+    goto connect_monitor;
 
   preferences->loading_in_progress = TRUE;
 
@@ -1525,6 +1523,7 @@ terminal_preferences_load (TerminalPreferences *preferences)
 
   g_object_thaw_notify (G_OBJECT (preferences));
 
+connect_monitor:
   /* startup file monitoring */
   terminal_preferences_monitor_connect (preferences, filename);
 
@@ -1763,7 +1762,7 @@ terminal_preferences_monitor_connect (TerminalPreferences *preferences,
         {
           /* connect signal */
 #ifndef NDEBUG
-          g_message ("Monitoring \"%s\" for changes.", filename);
+          g_debug ("Monitoring \"%s\" for changes.", filename);
 #endif
           g_signal_connect (G_OBJECT (preferences->monitor), "changed",
                             G_CALLBACK (terminal_preferences_monitor_changed),
@@ -1772,7 +1771,7 @@ terminal_preferences_monitor_connect (TerminalPreferences *preferences,
       else
         {
           g_critical ("Failed to setup monitoring for file \"%s\": %s",
-                       filename, error->message);
+                      filename, error->message);
           g_error_free (error);
         }
     }
@@ -1823,16 +1822,20 @@ terminal_preferences_get (void)
 
 
 
-void
+gboolean
 terminal_preferences_get_color (TerminalPreferences *preferences,
                                 const gchar         *property,
                                 GdkColor            *color_return)
 {
-  gchar *spec;
+  gchar    *spec;
+  gboolean  succeed = FALSE;
 
-  terminal_return_if_fail (TERMINAL_IS_PREFERENCES (preferences));
+  terminal_return_val_if_fail (TERMINAL_IS_PREFERENCES (preferences), FALSE);
 
   g_object_get (G_OBJECT (preferences), property, &spec, NULL);
-  gdk_color_parse (spec, color_return);
+  if (G_LIKELY (spec != NULL))
+    succeed = gdk_color_parse (spec, color_return);
   g_free (spec);
+
+  return succeed;
 }
