@@ -1,4 +1,3 @@
-/* $Id: terminal-screen.c 30351 2009-07-19 15:31:07Z nick $ */
 /*-
  * Copyright (c) 2004-2007 os-cillation e.K.
  *
@@ -47,6 +46,7 @@
 #include <terminal/terminal-private.h>
 #include <terminal/terminal-screen.h>
 #include <terminal/terminal-widget.h>
+#include <terminal/terminal-window.h>
 
 #if defined(GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
@@ -85,6 +85,8 @@ static gboolean   terminal_screen_get_child_command             (TerminalScreen
                                                                  gchar                **command,
                                                                  gchar               ***argv,
                                                                  GError               **error);
+static gchar     *terminal_screen_parse_title                   (TerminalScreen        *screen,
+                                                                 const gchar           *title);
 static gchar    **terminal_screen_get_child_environment         (TerminalScreen        *screen);
 static void       terminal_screen_update_background             (TerminalScreen        *screen);
 static void       terminal_screen_update_binding_backspace      (TerminalScreen        *screen);
@@ -111,6 +113,10 @@ static void       terminal_screen_vte_selection_changed         (VteTerminal
                                                                  TerminalScreen        *screen);
 static void       terminal_screen_vte_window_title_changed      (VteTerminal           *terminal,
                                                                  TerminalScreen        *screen);
+static void       terminal_screen_vte_resize_window             (VteTerminal           *terminal,
+                                                                 guint                  width,
+                                                                 guint                  height,
+                                                                 TerminalScreen        *screen);
 static void       terminal_screen_vte_window_contents_changed   (VteTerminal           *terminal,
                                                                  TerminalScreen        *screen);
 static gboolean   terminal_screen_timer_background              (gpointer               user_data);
@@ -132,13 +138,15 @@ struct _TerminalScreen
   GtkWidget           *scrollbar;
   GtkWidget           *tab_label;
 
+  guint                session_id;
+
   GPid                 pid;
   gchar               *working_directory;
 
   gchar              **custom_command;
   gchar               *custom_title;
 
-  gboolean             hold;
+  guint                hold : 1;
 
   guint                background_timer_id;
   guint                launch_idle_id;
@@ -150,6 +158,7 @@ struct _TerminalScreen
 
 
 static guint screen_signals[LAST_SIGNAL];
+static guint screen_last_session_id = 0;
 
 
 
@@ -223,6 +232,7 @@ static void
 terminal_screen_init (TerminalScreen *screen)
 {
   screen->working_directory = g_get_current_dir ();
+  screen->session_id = ++screen_last_session_id;
 
   screen->terminal = g_object_new (TERMINAL_TYPE_WIDGET, NULL);
   g_object_connect (G_OBJECT (screen->terminal),
@@ -231,6 +241,7 @@ terminal_screen_init (TerminalScreen *screen)
                     "signal::context-menu", G_CALLBACK (terminal_screen_vte_get_context_menu), screen,
                     "signal::selection-changed", G_CALLBACK (terminal_screen_vte_selection_changed), screen,
                     "signal::window-title-changed", G_CALLBACK (terminal_screen_vte_window_title_changed), screen,
+                    "signal::resize-window", G_CALLBACK (terminal_screen_vte_resize_window), screen,
                     "swapped-signal::size-allocate", G_CALLBACK (terminal_screen_timer_background), screen,
                     "swapped-signal::style-set", G_CALLBACK (terminal_screen_update_colors), screen,
                     NULL);
@@ -343,11 +354,14 @@ terminal_screen_get_property (GObject          *object,
 {
   TerminalScreen *screen = TERMINAL_SCREEN (object);
   const gchar    *title = NULL;
+  TerminalTitle   mode;
+  gchar          *initial = NULL;
+  gchar          *custom_title;
 
   switch (prop_id)
     {
     case PROP_CUSTOM_TITLE:
-      if (IS_STRING (screen->custom_title))
+      if (screen->custom_title != NULL)
         g_value_set_string (value, screen->custom_title);
       else
         g_value_set_static_string (value, "");
@@ -355,10 +369,30 @@ terminal_screen_get_property (GObject          *object,
 
     case PROP_TITLE:
       if (G_UNLIKELY (screen->custom_title != NULL))
-        title = screen->custom_title;
-      else if (G_LIKELY (screen->terminal != NULL))
-        title = vte_terminal_get_window_title (VTE_TERMINAL (screen->terminal));
-      g_value_set_string (value, title != NULL ? title : _("Untitled"));
+        {
+          custom_title = terminal_screen_parse_title (screen, screen->custom_title);
+          g_value_take_string (value, custom_title);
+        }
+      else
+        {
+          g_object_get (G_OBJECT (screen->preferences), "title-mode", &mode, NULL);
+          if (G_UNLIKELY (mode == TERMINAL_TITLE_HIDE))
+            {
+              /* show the initial title if the dynamic title is set to hidden */
+              g_object_get (G_OBJECT (screen->preferences), "title-initial", &initial, NULL);
+              title = initial;
+            }
+          else if (G_LIKELY (screen->terminal != NULL))
+            {
+              title = VTE_TERMINAL (screen->terminal)->window_title;
+            }
+
+          /* TRANSLATORS: title for the tab/window used when all other
+           * possible titles were empty strings */
+          g_value_set_string (value, title != NULL ? title : _("Untitled"));
+
+          g_free (initial);
+        }
       break;
 
     default:
@@ -486,6 +520,91 @@ terminal_screen_get_child_command (TerminalScreen   *screen,
 
 
 
+static gchar *
+terminal_screen_parse_title (TerminalScreen *screen,
+                             const gchar    *title)
+{
+  GString     *string;
+  const gchar *remainder;
+  const gchar *percent;
+  const gchar *directory = NULL;
+  gchar       *base_name;
+  const gchar *vte_title;
+
+  terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
+
+  if (G_UNLIKELY (title == NULL))
+    return g_strdup ("");
+
+  string = g_string_new (NULL);
+  remainder = title;
+
+  /* walk from % character to % character */
+  for (;;)
+    {
+      /* look out for the next % character */
+      percent = strchr (remainder, '%');
+      if (percent == NULL)
+        {
+          /* we parsed the whole string */
+          g_string_append (string, remainder);
+          break;
+        }
+
+      /* append the characters in between */
+      g_string_append_len (string, remainder, percent - remainder);
+      remainder = percent + 1;
+
+      /* handle the "%" character */
+      switch (*remainder)
+        {
+        case '#':
+          g_string_append_printf (string, "%u", screen->session_id);
+          break;
+
+        case 'd':
+        case 'D':
+          if (directory == NULL)
+            directory = terminal_screen_get_working_directory (screen);
+
+          if (G_LIKELY (directory != NULL))
+            {
+              if (*remainder == 'D')
+                {
+                  /* long directory name */
+                  g_string_append (string, directory);
+                }
+              else
+                {
+                  /* short directory name */
+                  base_name = g_path_get_basename (directory);
+                  g_string_append (string, base_name);
+                  g_free (base_name);
+                }
+            }
+          break;
+
+        case 'w':
+          /* window title from vte */
+          vte_title = VTE_TERMINAL (screen->terminal)->window_title;
+          if (G_UNLIKELY (vte_title == NULL))
+            vte_title = _("Untitled");
+          g_string_append (string, vte_title);
+          break;
+
+        default:
+          g_string_append_c (string, '%');
+          continue;
+        }
+
+      remainder++;
+    }
+
+  return g_string_free (string, FALSE);
+}
+
+
+
 static gchar**
 terminal_screen_get_child_environment (TerminalScreen *screen)
 {
@@ -631,28 +750,45 @@ terminal_screen_update_colors (TerminalScreen *screen)
   GdkColor fg;
   GdkColor cursor;
   GdkColor selection;
-  gboolean selection_use_default;
+  gboolean use_default;
   gchar    name[32];
   guint    n;
+  gboolean has_bg;
+  gboolean has_fg;
+  gboolean has_cursor;
+  gboolean has_pallette;
 
-  terminal_preferences_get_color (screen->preferences, "color-background", &bg);
-  terminal_preferences_get_color (screen->preferences, "color-foreground", &fg);
-  terminal_preferences_get_color (screen->preferences, "color-cursor", &cursor);
+  has_bg = terminal_preferences_get_color (screen->preferences, "color-background", &bg);
+  has_fg = terminal_preferences_get_color (screen->preferences, "color-foreground", &fg);
+  has_cursor = terminal_preferences_get_color (screen->preferences, "color-cursor", &cursor);
 
-  for (n = 0; n < 16; ++n)
+  for (n = 0, has_pallette = TRUE; has_pallette && n < 16; ++n)
     {
-      g_snprintf (name, 32, "color-palette%u", n + 1);
-      terminal_preferences_get_color (screen->preferences, name, palette + n);
+      g_snprintf (name, sizeof (name), "color-palette%u", n + 1);
+      has_pallette = terminal_preferences_get_color (screen->preferences, name, palette + n);
     }
 
-  vte_terminal_set_colors (VTE_TERMINAL (screen->terminal), &fg, &bg, palette, 16);
-  vte_terminal_set_background_tint_color (VTE_TERMINAL (screen->terminal), &bg);
-  vte_terminal_set_color_cursor (VTE_TERMINAL (screen->terminal), &cursor);
+  if (G_LIKELY (has_pallette))
+    {
+      vte_terminal_set_colors (VTE_TERMINAL (screen->terminal),
+                               has_fg ? &fg : NULL,
+                               has_bg ? &bg : NULL,
+                               palette, 16);
+    }
+  else
+    {
+      vte_terminal_set_default_colors (VTE_TERMINAL (screen->terminal));
+      g_warning ("One of the terminal colors (color-palette%u) was not parsed "
+                 "successfully. The default palette has been applied.", n);
+    }
 
-  g_object_get (G_OBJECT (screen->preferences), "color-selection-use-default", &selection_use_default, NULL);
-  if (!selection_use_default)
-    terminal_preferences_get_color (screen->preferences, "color-selection", &selection);
-  vte_terminal_set_color_highlight (VTE_TERMINAL (screen->terminal), selection_use_default ? NULL : &selection);
+  vte_terminal_set_background_tint_color (VTE_TERMINAL (screen->terminal), has_bg ? &bg : NULL);
+  vte_terminal_set_color_cursor (VTE_TERMINAL (screen->terminal), has_cursor ? &cursor : NULL);
+
+  g_object_get (G_OBJECT (screen->preferences), "color-selection-use-default", &use_default, NULL);
+  if (!use_default)
+    use_default = !terminal_preferences_get_color (screen->preferences, "color-selection", &selection);
+  vte_terminal_set_color_highlight (VTE_TERMINAL (screen->terminal), use_default ? NULL : &selection);
 }
 
 
@@ -906,6 +1042,48 @@ terminal_screen_vte_window_title_changed (VteTerminal    *terminal,
 
 
 
+static void
+terminal_screen_vte_resize_window (VteTerminal    *terminal,
+                                   guint           width,
+                                   guint           height,
+                                   TerminalScreen *screen)
+{
+  GtkWidget *toplevel;
+  gint       xpad;
+  gint       ypad;
+  gint       grid_width;
+  gint       grid_height;
+
+  terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
+  terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+
+  /* don't do anything if the window is already fullscreen/maximized */
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
+  if (!GTK_WIDGET_REALIZED (toplevel)
+      || (gdk_window_get_state (toplevel->window)
+          & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) != 0)
+    return;
+
+  /* we have to calculate the grid size, because the signal
+   * returns a size in pixels */
+  vte_terminal_get_padding (terminal, &xpad, &ypad);
+  grid_width = (width - xpad) / terminal->char_width;
+  grid_height = (height - ypad) / terminal->char_height;
+
+  /* leave if there is nothing to resize */
+  if (terminal->column_count == grid_width
+      && terminal->row_count == grid_height)
+    return;
+
+  /* set the terminal size and resize the window if it is active */
+  vte_terminal_set_size (terminal, grid_width, grid_height);
+  if (screen == terminal_window_get_active (TERMINAL_WINDOW (toplevel)))
+    terminal_screen_force_resize_window (screen, GTK_WINDOW (toplevel),
+                                         grid_width, grid_height);
+}
+
+
+
 static gboolean
 terminal_screen_reset_activity_timeout (gpointer user_data)
 {
@@ -934,6 +1112,7 @@ terminal_screen_vte_window_contents_changed (VteTerminal    *terminal,
 {
   guint    timeout;
   GdkColor color;
+  gboolean has_fg;
 
   terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
@@ -952,8 +1131,8 @@ terminal_screen_vte_window_contents_changed (VteTerminal    *terminal,
     return;
 
   /* set label color */
-  terminal_preferences_get_color (screen->preferences, "tab-activity-color", &color);
-  gtk_widget_modify_fg (screen->tab_label, GTK_STATE_ACTIVE, &color);
+  has_fg = terminal_preferences_get_color (screen->preferences, "tab-activity-color", &color);
+  gtk_widget_modify_fg (screen->tab_label, GTK_STATE_ACTIVE, has_fg ? &color : NULL);
 
   /* stop running reset timeout */
   if (screen->activity_timeout_id != 0)
@@ -976,7 +1155,8 @@ terminal_screen_timer_background (gpointer user_data)
   TerminalBackground   background_mode;
   GdkPixbuf           *image;
   gdouble              background_darkness;
-  gboolean             transparent = FALSE;
+  gdouble              saturation = 1.0;
+  guint16              opacity = 0xffff;
 
   terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
   terminal_return_val_if_fail (VTE_IS_TERMINAL (screen->terminal), FALSE);
@@ -985,7 +1165,6 @@ terminal_screen_timer_background (gpointer user_data)
 
   g_object_get (G_OBJECT (screen->preferences), "background-mode", &background_mode, NULL);
 
-
   if (G_UNLIKELY (background_mode == TERMINAL_BACKGROUND_IMAGE))
     {
       loader = terminal_image_loader_get ();
@@ -1008,25 +1187,15 @@ terminal_screen_timer_background (gpointer user_data)
     {
       g_object_get (G_OBJECT (screen->preferences), "background-darkness", &background_darkness, NULL);
 
-      if (gtk_widget_is_composited (GTK_WIDGET (screen)))
-        {
-          vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), 0xffff * background_darkness);
-          vte_terminal_set_background_saturation (VTE_TERMINAL (screen->terminal), 1.0);
-        }
-      else
-        {
-          vte_terminal_set_background_saturation (VTE_TERMINAL (screen->terminal), 1.0 - background_darkness);
-          vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), 0xffff);
-          transparent = TRUE;
-        }
-    }
-  else
-    {
-      vte_terminal_set_background_saturation (VTE_TERMINAL (screen->terminal), 1.0);
-      vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), 0xffff);
+      saturation = 1.0 - background_darkness;
+      opacity = 0xffff * background_darkness;
     }
 
-  vte_terminal_set_background_transparent (VTE_TERMINAL (screen->terminal), transparent);
+  vte_terminal_set_background_saturation (VTE_TERMINAL (screen->terminal), saturation);
+  vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), opacity);
+  vte_terminal_set_background_transparent (VTE_TERMINAL (screen->terminal),
+                                           background_mode == TERMINAL_BACKGROUND_TRANSPARENT
+                                           && !gtk_widget_is_composited (GTK_WIDGET (screen)));
 
   screen->last_size_change = time (NULL);
 
@@ -1113,7 +1282,7 @@ terminal_screen_launch_child (TerminalScreen *screen)
 
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
-#ifdef G_ENABLE_DEBUG
+#ifndef NDEBUG
   if (!GTK_WIDGET_REALIZED (screen))
     g_error ("Tried to launch command in a TerminalScreen that is not realized");
 #endif
@@ -1251,7 +1420,7 @@ terminal_screen_set_window_geometry_hints (TerminalScreen *screen,
 
   hints.base_width = xpad;
   hints.base_height = ypad;
-  hints.width_inc = VTE_TERMINAL (screen->terminal)->char_width;
+  hints.width_inc = VTE_TERMINAL (screen->terminal)->char_width ;
   hints.height_inc = VTE_TERMINAL (screen->terminal)->char_height;
   hints.min_width = hints.base_width + hints.width_inc * 4;
   hints.min_height = hints.base_height + hints.height_inc * 2;
@@ -1310,7 +1479,6 @@ terminal_screen_force_resize_window (TerminalScreen *screen,
     rows = force_rows;
 
   vte_terminal_get_padding (VTE_TERMINAL (screen->terminal), &xpad, &ypad);
-
   width += xpad + VTE_TERMINAL (screen->terminal)->char_width * columns;
   height += ypad + VTE_TERMINAL (screen->terminal)->char_height * rows;
 
@@ -1337,26 +1505,29 @@ terminal_screen_get_title (TerminalScreen *screen)
 {
   TerminalTitle  mode;
   const gchar   *vte_title;
-  gchar         *initial;
+  gchar         *initial, *tmp;
   gchar         *title;
 
   terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
 
   if (G_UNLIKELY (screen->custom_title != NULL))
-    return g_strdup (screen->custom_title);
+    return terminal_screen_parse_title (screen, screen->custom_title);
 
+  vte_title = VTE_TERMINAL (screen->terminal)->window_title;
   g_object_get (G_OBJECT (screen->preferences),
-                "title-initial", &initial,
                 "title-mode", &mode,
+                "title-initial", &tmp,
                 NULL);
-
-  vte_title = vte_terminal_get_window_title (VTE_TERMINAL (screen->terminal));
+  initial = terminal_screen_parse_title (screen, tmp);
+  g_free (tmp);
 
   switch (mode)
     {
     case TERMINAL_TITLE_REPLACE:
       if (G_LIKELY (vte_title != NULL))
         title = g_strdup (vte_title);
+      else if (initial != NULL)
+        return initial;
       else
         title = g_strdup (_("Untitled"));
       break;
@@ -1365,18 +1536,18 @@ terminal_screen_get_title (TerminalScreen *screen)
       if (G_LIKELY (vte_title != NULL))
         title = g_strconcat (vte_title, " - ", initial, NULL);
       else
-        title = g_strdup (initial);
+        return initial;
       break;
 
     case TERMINAL_TITLE_APPEND:
       if (G_LIKELY (vte_title != NULL))
         title = g_strconcat (initial, " - ", vte_title, NULL);
       else
-        title = g_strdup (initial);
+        return initial;
       break;
 
     case TERMINAL_TITLE_HIDE:
-      title = g_strdup (initial);
+      return initial;
       break;
 
     default:
@@ -1469,25 +1640,6 @@ terminal_screen_set_working_directory (TerminalScreen *screen,
 
 
 
-/**
- * terminal_screen_get_hold:
- * @screen : A #TerminalScreen.
- *
- * Checks whether the terminal screen will be destroyed when
- * the child exits.
- *
- * Return value: %TRUE if @screen will be destroyed once
- *               it's child exits.
- **/
-gboolean
-terminal_screen_get_hold (TerminalScreen *screen)
-{
-  terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
-  return screen->hold;
-}
-
-
-
 /**
  * terminal_screen_set_hold:
  * @screen : A #TerminalScreen.
@@ -1685,7 +1837,8 @@ terminal_screen_get_tab_label (TerminalScreen *screen)
   gtk_misc_set_padding (GTK_MISC (screen->tab_label), 2, 0);
   gtk_box_pack_start  (GTK_BOX (hbox), screen->tab_label, TRUE, TRUE, 0);
   exo_binding_new (G_OBJECT (screen), "title", G_OBJECT (screen->tab_label), "label");
-  exo_binding_new (G_OBJECT (screen), "title", G_OBJECT (screen->tab_label), "tooltip-text");
+  exo_binding_new (G_OBJECT (screen->tab_label), "label",
+                   G_OBJECT (screen->tab_label), "tooltip-text");
   gtk_widget_set_has_tooltip (screen->tab_label, TRUE);
   gtk_widget_show (screen->tab_label);
 
